#include <iostream>

#include "forward_list.h"

struct sleeping_dog_list_tag;
struct black_dog_list_tag;
struct red_cat_list_tag;
struct doge;

using sleeping_dog_list_element = atmos::container::forward_list_element_tagged<sleeping_dog_list_tag, doge>;
using black_dog_list_element = atmos::container::forward_list_element_tagged<black_dog_list_tag, doge>;
using red_cat_list_element = atmos::container::forward_list_element_tagged<red_cat_list_tag, doge>;

using sleeping_dog_list = atmos::container::forward_list_tagged<sleeping_dog_list_element>;
using black_dog_list = atmos::container::forward_list_tagged<black_dog_list_element>;
using red_cat_list = atmos::container::forward_list_tagged<red_cat_list_element>;

struct doge : sleeping_dog_list_element, black_dog_list_element
{
	doge(const char* name) noexcept
		: name(name)
	{
	}

	const char* name;
};

sleeping_dog_list sleeping_dogs;
black_dog_list black_dogs;
red_cat_list red_cats;

template<typename List>
void print_list(List& list)
{
	auto current = list.first();
	const char* comma = "";
	while (current)
	{
		std::cout << comma;
		comma = ", ";
		std::cout << (*current)->name;
		current = List::next(current);
	}
}

void print_dogs()
{
	std::cout << "Sleeping dogs: ";
	print_list(sleeping_dogs);
	std::cout << std::endl << "Black dogs: ";
	print_list(black_dogs);
	std::cout << std::endl;
}

int main()
{
	doge dogs[5]{ "Buster", "Shawn1x", "Roxy", "Ginger", "Ruby" };

	sleeping_dogs.push_front(&dogs[0]);
	sleeping_dogs.push_front(&dogs[1]);
	sleeping_dogs.push_front(&dogs[2]);

	black_dogs.push_front(&dogs[2]);
	black_dogs.push_front(&dogs[3]);
	black_dogs.push_front(&dogs[4]);

	print_dogs();

	std::cout << std::endl << dogs[4].name << " is now sleeping, too!" << std::endl;
	sleeping_dogs.push_front(&dogs[4]);
	
	std::cout << dogs[3].name << " is not black anymore!" << std::endl;
	black_dogs.remove(&dogs[3]);

	print_dogs();
}
